# HSPECFIT1 - Subtract rescaled mean spectrum from spectrum
# November 27, 2006
# By Doug Mink, SAO Telescope Data Center

procedure hspecfit1 (objfile, fitfile, outfile)

file	objfile=""	{prompt="Spectrum file"}
file	objap=""	{prompt="Aperture of spectrum to fit"}
file	objdir=""	{prompt="Directory containing object+sky spectrum"}
file	fitfile=""	{prompt="Fit spectrum file"}
file	fitaps=""	{prompt="Range of apertures to average and fit"}
file	fitdir=""	{prompt="Directory containing spectra to fit"}
string	combine="average"	{prompt="Combine fitaps as average or median or sum"}
file	reject="none"	{prompt="Rejection operation for scombine"}
file	outfile=""	{prompt="Output residual spectrum file"}
file	outdir=""	{prompt="Directory for output spectrum"}
bool	interact=no	{prompt="Interact with mean spectrum (yes or no)"}
bool	plotres=no	{prompt="Plot result (yes or no)"}
bool	verbose=no	{prompt="Log progress (yes or no)"}
bool	debug=no	{prompt="Print debugging information (yes or no)"}

begin
	file ims, imin, fname, inspec, outspec, meanfile
	bool vb

	inspec = objfile
	outspec = outfile
	vb = verbose

#	if (!defpac ("rvsao")) {
#	    rvsao
#	    }

	meanfile = mktemp ("specfit")
	scombine.input = fitfile
	scombine.output = meanfile
	scombine.noutput = "1"
	scombine.logfile = "STDOUT"
	scombine.apertures = fitaps
	scombine.group = "all"
	scombine.combine = combine
	scombine.reject = reject
	scombine.first = yes
	scombine.w1 = INDEF
	scombine.w2 = INDEF
	scombine.dw = INDEF
	scombine.nw = INDEF
	scombine.log = no
	scombine.scale = "none"
	scombine.zero = "none"
	scombine.weight = "none"
	scombine.sample = ""
	scombine.lthreshold = 0.000001
	scombine.hthreshold = INDEF
	scombine.nlow = 1
	scombine.nhigh = 1
	scombine.nkeep = 1
	scombine.mclip = no
	scombine.lsigma = 3.
	scombine.hsigma = 3.
	scombine.rdnoise = "0."
	scombine.gain = "1."
	scombine.snoise = "0."
	scombine.sigscale = 0.1
	scombine.pclip = -0.5
	scombine.grow = 0
	scombine.blank = 0.
	scombine (fitfile, meanfile)

#	sumspec.specnum = fitaps
#	sumspec.specband = 0
#	sumspec.specdir = fitdir
#	sumspec.compname = ""
#	sumspec.compdir = ""
#	sumspec.nspec = 1
#	sumspec.save_names = yes
#	sumspec.copy_header = yes
#	sumspec.normin = 0.0
#	sumspec.fixbad = "no"
#	sumspec.badlines = "badskyab.dat"
#	sumspec.linedir = "hectospec$lib/"
#	sumspec.cont_remove = "no"
#	sumspec.cont_split = 1
#	sumspec.reject = no
#	sumspec.abs_reject = 100.
#	sumspec.em_reject = 2.
#	sumspec.contout = "no"
#	sumspec.cont_plot = no
#	sumspec.cont_add = 0.
#	sumspec.spec_smooth = 0
#	sumspec.st_lambda = 1.
#	sumspec.end_lambda = INDEF
#	sumspec.pix_lambda = 1.
#	sumspec.npts = 1
#	sumspec.complog = no
#	sumspec.interp_mode = "spline3"
#	sumspec.normout = 0.0
#	sumspec.spec_plot = no
#	sumspec.spec_int = no
#	sumspec.comp_plot = vb
#	sumspec.comp_int = interact
#	sumspec.ymin = 0.5
#	sumspec.ymax = 1.5
#	sumspec.velcomp = INDEF
#	sumspec.zcomp = INDEF
#	sumspec.svel_corr = "none"
#	sumspec.nsmooth = 0
#	sumspec.device = "stdgraph"
#	sumspec.plotter = "stdplot"
#	sumspec.logfiles = "STDOUT,specfit.log"
#	sumspec.nsum = 1
#	sumspec.verbose = vb
#	sumspec.debug = debug
#	sumspec (fitfile, meanfile, compdir="")

	if (!defpac ("svdfit")) {
	    svdfit
	    }

#  Set up parameters to fit mean spectrum to object spectrum
	contres.c_interactive = no
	contres.c_sample = "*"
	contres.c_function = "spline3"
	contres.naverage = 1
	contres.order = 15
	contres.low_reject = 2.
	contres.high_reject = 2.
	contres.niterate = 10
	contres.grow = 1.

	svdres.spectra = inspec
	svdres.specnum = objap
	svdres.specband = 1
	svdres.specdir = objdir
	svdres.eigenfile = meanfile
	svdres.eigenum = "1"
	svdres.eigenband = 1
	svdres.eigendir = fitdir
	svdres.eigentrack = no
	svdres.resfile = outspec
	svdres.resdir = outdir
	svdres.res_format = "multispec"
	svdres.st_lambda = INDEF
	svdres.end_lambda = INDEF
	svdres.nbin = INDEF
	svdres.interp_mode = "spline3"
	svdres.obj_plot = no
	svdres.vec_plot = no
	svdres.res_plot = plotres
	svdres.plot_int = no
	svdres.fixbad = "fill"
	svdres.badlines = "badlines.dat"
	svdres.cont_remove = "subtract"
	svdres.vec_cont = "subtract"
	svdres.cont_add = no
	svdres.fitmax = 1.5
	svdres.logfiles = "STDOUT,svdres.log"
	svdres.nsmooth = 0
	svdres.ablines = "ablines.dat"
	svdres.emlines = "emlines.dat"
	svdres.linedir = "svdfit$lib/"
	svdres.dispmode = 1
	svdres.displot = yes
	svdres.device = "stdgraph"
	svdres.curmode = no
	svdres.contsub_plot = no
	svdres.verbose = vb
	svdres.debug = debug
	svdres.nsum = 1

#	dpar ("svdres",>"svdres.par")
	svdres (inspec,outspec)

	if (vb) {
	    print (inspec," local mean spectrum removed using SVDRES")
	    }
end

# Nov 27 2006	New task based on skyfit1.cl
