# File hectospec/Scripts/gaincorr.cl
# By Bill Wyatt, with revisions by Doug Mink
# March 5, 2008

# Run the 4 separate gain corrections for HECTOSPEC
# Input may be a comma-separated list or a filename (with prefaced "@")
# If output list not given, the updates are in place, else the output
# list must match the input in number.

procedure gaincorr (input, output)

string	input = ""	{prompt="Images to correct"}
string	output= ""	{prompt="Output image"}

string	g1    = "!GAIN"	{prompt="Gain for amplifier 1"}
string	g2    = "!GAIN"	{prompt="Gain for amplifier 2"}
string	g3    = "!GAIN"	{prompt="Gain for amplifier 3"}
string	g4    = "!GAIN"	{prompt="Gain for amplifier 4"}

bool   hupdate = yes	{prompt="Check header for GAINCORR and record same"}
bool   vb      = yes	{prompt="verbose output (y or n)"}

struct *infile
struct *outfile

begin

   string inlist, outlist
   string tchar, val, val0
   string gcsect1, gcsect2, gcsect3, gcsect4
   string imsect1, imsect2, imsect3, imsect4
   string procid, gainkey
   string fname
   file   imagei, imageo
   real gain1, gain2, gain3, gain4

   inlist  = input
   outlist = output
   
# see if output is null ==> gain correct file(s) in place

   procid = mktemp("tmp$")   # really the procid plus a character

   tchar = substr(inlist, 1, 1)
   if(tchar == "@") {
      infile = substr(inlist,2,100)
   }
   else {
     fname = procid // "gcin"
     infile = fname
     files(inlist, >fname)
   }

   tchar = substr(outlist, 1, 1)
   fname = procid // "gcout"
   outfile = fname
   if(tchar == "@") {
      outfile = substr(outlist,2,100)
   }
   else if (outlist == "") {
     files(inlist, >fname)
   } else {
     files(outlist, >fname)
   }    

   while( fscan(infile, imagei) != EOF) {
     if(fscan(outfile, imageo) == EOF) {
        printf("GAINCORR: output list too short (input=%s)?\n", imagei)
        break
     }
     if(vb) printf("GAINCORR: imarith on extns of %s to %s\n",
          str(imagei), str(imageo))

  # check if GAINCORRection has already been done
#  If amplifier gain has already been done, return
    imgetp.default = "0"
    imsect1 = imagei // "[1]"
    imsect2 = imagei // "[2]"
    imsect3 = imagei // "[3]"
    imsect4 = imagei // "[4]"
    imgetp (imsect1, "GAINCORR")
    if (imgetp.value != "0") {
	print ("GAINCORR: GAINCORR has already been run on", imagei)
	print ("GAINCORR: ", imgetp.value)
	imgetp (imsect2, "GAINCORR")
	print ("GAINCORR: ", imgetp.value)
	imgetp (imsect3, "GAINCORR")
	print ("GAINCORR: ", imgetp.value)
	imgetp (imsect4, "GAINCORR")
	print ("GAINCORR: ", imgetp.value)
	}


  # or, if HUPDATE is false, do it anyway, but no recording of header
     if (imgetp.value != "0" && hupdate) {
       printf("skipping %s: already gain corrected\n", imagei)

       # if output is different, copy it over
       if(imagei != imageo) {
         if(access(imageo)) { 
             imdelete(imageo) 
         } else if(access(imageo // ".fits")) {
             imdelete(imageo // ".fits") 
         } else if(access(imageo // ".fit")) {
             imdelete(imageo // ".fit") 
         } else if(access(imageo // ".fts")) {
             imdelete(imageo // ".fts") 
         } else if(access(imageo // ".imh")) {
             imdelete(imageo // ".imh") 
         }
         msccmd("imcopy $input $output verbose=" // vb, imagei, imageo)
       }
       next  # skip to beginning of loop
     }

# Get gain for each extension of image
	if (g1 == "") {
	    gain1 = 1.0
	    }
	else if (substr (g1, 1, 1) == "!") {
	    gainkey = substr (g1, 2, 9)
	    imgetp.default = "0" 
	    imgetp (imsect1, gainkey)
	    if (imgetp.value == "0") {
		print ("HGAIN1: Gain 1 keyword ", gainkey, " not found")
		print ("HGAIN1: No gain correction can be done")
		gain2 = 1.0
		}
	    else
		gain1 = real (imgetp.value)
	    }
	else {
	    gain1 = real (g1)
	    }

	if (g2 == "") {
	    gain2 = 1.0
	    }
	else if (substr (g2, 1, 1) == "!") {
	    gainkey = substr (g2, 2, 9)
	    imgetp.default = "0" 
	    imgetp (imsect2, gainkey)
	    if (imgetp.value == "0") {
		print ("HGAIN1: Gain 2 keyword ", gainkey, " not found")
		print ("HGAIN1: No gain correction can be done")
		gain2 = 1.0
		}
	    else
		gain2 = real (imgetp.value)
	    }
	else {
	    gain2 = real (g2)
	    }

	if (g3 == "") {
	    gain3 = 1.0
	    }
	else if (substr (g3, 1, 1) == "!") {
	    gainkey = substr (g3, 2, 9)
	    imgetp.default = "0" 
	    imgetp (imsect3, gainkey)
	    if (imgetp.value == "0") {
		print ("HGAIN1: Gain 3 keyword ", gainkey, " not found")
		print ("HGAIN1: No gain correction can be done")
		gain3 = 1.0
		}
	    else
		gain3 = real (imgetp.value)
	    }
	else {
	    gain3 = real (g3)
	    }

	if (g4 == "") {
	    gain4 = 0.0
	    }
	else if (substr (g4, 1, 1) == "!") {
	    gainkey = substr (g4, 2, 9)
	    imgetp.default = "0" 
	    imgetp (imsect4, gainkey)
	    if (imgetp.value == "0") {
		print ("HGAIN1: Gain 4 keyword ", gainkey, " not found")
		print ("HGAIN1: No gain correction can be done")
		gain4 = 0.0
		}
	    else
		gain4 = real (imgetp.value)
	    }
	else {
	    gain4 = real (g4)
	    }


  #otherwise, do the corrections
     imcopy (imagei // "[0]", procid // "GC_0", verbose=vb)

     gcsect1 = procid // "GC_1"
     gcsect2 = procid // "GC_2"
     gcsect3 = procid // "GC_3"
     gcsect4 = procid // "GC_4"

     imarith(imagei // "[1]", '/', gain1, gcsect1 ) # , verbose=vb)
     imarith(imagei // "[2]", '/', gain2, gcsect2 ) # , verbose=vb)
     imarith(imagei // "[3]", '/', gain3, gcsect3 ) # , verbose=vb)
     imarith(imagei // "[4]", '/', gain4, gcsect4 ) # , verbose=vb)

  # Add GAINCORR keyword to header
     if (hupdate) {
       hedit.show = no
       gettime ()
       val0 = gettime.date // " " // gettime.time
       val = val0 // " Corrected amp 1 for gain of " // gain1
       hedit (gcsect1, "GAINCORR", val, add=yes, verify=no)
       hedit (gcsect1, "GAIN", "1.0", update=yes, add=yes, verify=no)
       if (vb)
          print ("GAINCORR: ",val)
       val = val0 // " Corrected amp 2 for gain of " // gain2
       hedit (gcsect2, "GAINCORR", val, add=yes, verify=no)
       hedit (gcsect2, "GAIN", "1.0", update=yes, add=yes, verify=no)
       if (vb)
          print ("GAINCORR: ",val)
       val = val0 // " Corrected amp 3 for gain of " // gain3
       hedit (gcsect3, "GAINCORR", val, add=yes, verify=no)
       hedit (gcsect3, "GAIN", "1.0", update=yes, add=yes, verify=no)
       if (vb)
          print ("GAINCORR: ",val)
       val = val0 // " Corrected amp 4 for gain of " // gain4
       hedit (gcsect4, "GAINCORR", val, add=yes, verify=no)
       hedit (gcsect4, "GAIN", "1.0", update=yes, add=yes, verify=no)
       if (vb)
          print ("GAINCORR: ",val)
     }

     if(access(imageo)) { 
       imdelete(imageo) 
     } else if(access(imageo // ".fits")) {
       imdelete(imageo // ".fits") 
     } else if(access(imageo // ".fit")) {
       imdelete(imageo // ".fit") 
     } else if(access(imageo // ".fts")) {
       imdelete(imageo // ".fts") 
     } else if(access(imageo // ".imh")) {
       imdelete(imageo // ".imh") 
     }
     if(vb) print("GAINCORR: joining ",imageo)
  # assemble the output file
     mscjoin(procid // "GC", output = imageo, delete=yes, verbose=vb)
   }

   delete(procid // "*")
   infile = ""
   outfile = ""

end
# Oct 27 2003	Bill Wyatt new task

# Mar  5 2008	Doug Mink read gains from header or parameter
# Mar  5 2008	Doug Mink divide rather than multiply by gains
# Mar  5 2008	Doug Mink revise GAINCORR keywords and delete GAIN from output
