# HMERGE1 - Combine 4 amps into one Hectospec image or two Hectochelle images
# February 22, 2008
# By Doug Mink, SAO Telescope Data Center

procedure hmerge1 (image)

string	image=""	{prompt="Image file to merge"}

bool	perchip=yes	{prompt="Merge amps to one output file per CCD (y or n)"}
bool	saveold=yes	{prompt="Save unmerged files in Unmerged/ (y or n)"}
bool	deltemp=yes	{prompt="Delete intermediate image files (y or n)"}
bool	verbose=yes	{prompt="Print process progress (y or n)"}

begin

file	imroot, temp1, temp2, temp3, temp4, im4, imin, templist, imout
file	imfile, newfile, imex0
string	amp1, amp2, amp3, amp4, arg1, imx, history
string	gain1, gain2, gain3, gain4
bool	dellist, vb, perc

	vb = verbose
	imx = ".fits"
	imin = image
	perc = perchip

	if (imin == "") {
	    print ("HMERGE1: Combine 4 amps on chips into 1 Hectospec image or 2 Hectochelle images")
	    print ("Usage: hmerge1 <input image with 4 extensions>")
	    return
	    }

	hedit.add = no
	hedit.addonly = no
	hedit.delete = no
	hedit.verify = no
	hedit.show = no
	hedit.update = yes

# Do not merge amplifiers twice
	imgetp.default = "0"
	imex0 = imin // "[0]"
	imgetp (imex0, "HMERGE")
	if (imgetp.value != "0") {
	    if (vb) {
		print ("HMERGE1: ", imin, " has already been merged")
		print ("HMERGE = ", imgetp.value)
		}
	    return
	    }

# Find root name of image
	imroot = imin
	smatch (imroot, imx)
	if (smatch.found) {
	    imroot = substr (imroot,1,smatch.start-1)
	    }
	else {
	    imroot = imageout
	    }
	if (vb) {
	    print ("HMERGE1: image root is ",imroot)
	    }

#  Split out one image per amplifier
	temp1 = mktemp ("hamp1") // imx
	amp1 = imroot // "[1]"
	imcopy (amp1, temp1)
	temp2 = mktemp ("hamp2") // imx
	amp2 = imroot // "[2]"
	imcopy (amp2, temp2)
	temp3 = mktemp ("hamp3") // imx
	amp3 = imroot // "[3]"
	imcopy (amp3, temp3)
	temp4 = mktemp ("hamp4") // imx
	amp4 = imroot // "[4]"
	imcopy (amp4, temp4)

# Get gain correction terms for output header(s)
	imgetp.default = "1.0"
	imgetp (amp1, "GAINCORR")
	gain1 = imgetp.value
	imgetp.default = "1.0"
	imgetp (amp2, "GAINCORR")
	gain2 = imgetp.value
	imgetp.default = "1.0"
	imgetp (amp3, "GAINCORR")
	gain3 = imgetp.value
	imgetp.default = "1.0"
	imgetp (amp4, "GAINCORR")
	gain4 = imgetp.value

# Move original file to Unflat directory
	if (saveold) {
	    if (!access ("Unmerged/")) {
		mkdir ("Unmerged")
		}
	    newfile = "Unmerged/" // imin
	    if (access (newfile)) {
		imdelete (newfile)
		}
	    movefiles (imin, "Unmerged")
	    if (vb) {
		print ("HMERGE1: Original ", imin, " => Unmerged/")
		}
	    }
	else {
	    delete (imin)
	    }
        gettime ()

#  Join two amplifiers per chip into two images
	if (perc) {
	    arg1 = temp1 // "," // temp2 // "[-*,*]"
	    imout = imroot // "_chip1" // imx
	    imjoin (arg1, imout, 1)

# Add gain corrections from each amplifier
	    hedit (imout, "GAINCOR1", gain1, add+)
	    hedit (imout, "GAINCOR2", gain2, add+)

# Make history line including time for preamp correction
	    hedit (imout, "GAINCORR", "", add-, delete+)
	    history = gettime.date // " " // gettime.time // " Amplifiers 1 and 2 merged into one image"
	    hedit (imout, "HMERGE", history, add+, delete-)

#  Join second chip amplifiers into a second file
	    arg1 = temp3 // "," // temp4 // "[-*,*]"
	    imout = imroot // "_chip2" // imx
	    imjoin (arg1, imout, 1)

# Add gain corrections from each amplifier
	    hedit (imout, "GAINCORR", "", add-, delete+)
	    hedit (imout, "GAINCOR3", gain3, add+, delete-)
	    hedit (imout, "GAINCOR4", gain4, add+, delete-)

# Make history line including time for preamp correction
	    history = gettime.date // " " // gettime.time // " Amplifiers 3 and 4 merged into one image"
	    hedit (imout, "HMERGE", history, add+, delete-)
	    }

#  Join two chips into a single image
	else {
	    arg1 = temp1 // "," // temp2 // "[-*,*]," // temp3 // "," // temp4 // "[-*,*]"
	    imjoin (arg1, imin, 1)

# Add gain corrections from each amplifier
	    hedit (imin, "GAINCORR", "", add-, delete+)
	    hedit (imin, "GAINCOR1", gain1, add+, delete-)
	    hedit (imin, "GAINCOR2", gain2, add+, delete-)
	    hedit (imin, "GAINCOR3", gain3, add+, delete-)
	    hedit (imin, "GAINCOR4", gain4, add+, delete-)

# Make history line including time for preamp correction
	    history = gettime.date // " " // gettime.time // " 4 amplifiers merged into one image"
	    hedit (imin, "HMERGE", history, add+, delete-)
	    }

	if (deltemp) {
	    imdelete (temp1)
	    imdelete (temp2)
	    imdelete (temp3)
	    imdelete (temp4)
	    }
end

# Feb 22 2008	New program for single image from HMERGE
