# HPIXSHIFT1 - Use emission line to find multispec spectra shift
# September 16, 2008
# By Doug Mink, SAO Telescope Data Center

procedure hpixshift1 (image)

string	image=""	{prompt="Image file for which to compute shifts"}

int	shiftline=1	{prompt="Line to shift in position file"}
file	emlines="hectosky.dat"	{prompt="File of emission line positions"}
file	linedir="rvsao$lib/"	{prompt="Directory for emission line positions"}
string	aprange="1-300" {prompt="Apertures over which to average pixel shift"}
bool	median=yes	{prompt="Set median line shift if yes, mean if no)"}
bool	saveshift=yes	{prompt="Save shift in header (yes or no)"}
bool	saveold = yes	{prompt="Save original files in Noshift/ (y or n)"}
#bool	curmode = no	{prompt="Interact with each spectrum (y or n)"}
bool	deltemp=yes	{prompt="Delete intermediate image files (y or n)"}
bool	verbose=yes	{prompt="Print shifts (y or n)"}
bool	debug=no	{prompt="Print debugging information (y or n)"}

struct *list1

begin

file	linefile, linepath
file	imin, newfile
int	sl
string	history
string	arg, cshift, lname
real	shift, wl, wl1, wl2, dwl
bool	vb, db, save

	vb = verbose
	db = debug
	save = saveold
	imin = image
	sl = shiftline
	linefile = emlines

	if (imin == "") {
	    print ("HPIXSHIFT1: Find pixel shift in multispec night sky spectra")
	    print ("           Run this task on a file which has been dispersion-corrected,")
	    print ("           but not linearized, where pixels are not linear in wavelength.")
	    print ("Usage: hpixshift1 <input image>")
	    return
	    }

# Return if this file already has shift set in header
	imgetp.default = "0"
	imgetp (imin, "HPIXSHIF")
	if (imgetp.value != "0") {
	    print ("HPIXSHIFT1 has already been run.")
	    print ("HPIXSHIFT: ", imgetp.value)
	    return
	    }

#  Figure out which line to check
	if (sl < 1) {
	    print ("HPIXSHIFT1: Wrong line number ",sl)
	    return
	    }

#  Load rvsao package to get emsao task
	if (!deftask ("emsao")) {
	    rvsao (motd=no)
	    }

#  Load wcstools package to get getcol task
	if (!defpac ("wcstools")) {
	    wcstools (motd=no)
	    }

# If possible, use single line file (pemsao only saves first line info)
	if (linefile == "hectosky.dat" && linedir == "hectospec$lib/") {
	    if (sl == 1)
		linefile = "O5577.dat"
	    else if (sl == 2)
		linefile = "O6300.dat"
	    else if (sl == 3)
		linefile = "OH7341.dat"
	    else if (sl == 4)
		linefile = "OH8399.dat"
	    else if (sl == 5)
		linefile = "OH8493.dat"
	    else {
		print ("HPIXSHIFT1: Wrong line number ", sl)
		return
		}
	    sl = 1
	    }
	linepath = linedir // linefile

# Find line name in line list file
	list1 = linepath
	for (i = 0; i < sl; i=i+1) {
	    if (fscan (list1, wl, wl1, wl2, dwl, lname) == EOF) {
		print ("HPIXSHIFT1: Fewer than ",sl," lines available")
		return
		}
#	    print (i,wl,lname)
	    }
	if (vb) {
	    print ("HPIXSHIFT1: Shift measured from ", lname)
	    }

#  Set up emsao parameters
	pemsao.spectra = imin
	pemsao.specnum = aprange
	pemsao.specband = 1
	pemsao.skynum = ""
	pemsao.skyband = 0
	pemsao.specdir = ""
	pemsao.linefit = yes
	pemsao.fixbad = no
	pemsao.badlines = "badlines.dat"
	pemsao.renormalize = no
	pemsao.st_lambda = INDEF
	pemsao.end_lambda = INDEF
	pemsao.nsmooth = 0
	pemsao.vel_init = "guess"
	pemsao.czguess = 0.
	pemsao.cortemp = ""
	pemsao.wspan = 10.
	pemsao.linesig = 1.
	pemsao.emsearch = ""
	pemsao.emlines = linefile
	pemsao.linedir = linedir
	pemsao.npfit = 2
	pemsao.nlcont = 1
	pemsao.esmooth = 2
	pemsao.emcombine = ""
	pemsao.mincont = 0.
	pemsao.lwmin = 0.4
	pemsao.lwmax = 1.7
	pemsao.lsmin = 1.
	pemsao.sigline = 0.
	pemsao.disperr = 0.01
	pemsao.vel_corr = "none"
	pemsao.report_mode = 11
	pemsao.archive = no
	pemsao.save_vel = no
	pemsao.verbose = yes
	pemsao.device = "stdgraph"
	pemsao.hardcopy = no
	if (vb && db) {
	    pemsao.displot = yes
	    }
	else {
	    pemsao.displot = no
	    }
        if (db) {
            pemsao.logfiles = "STDOUT,pemsao.log"
            }
        else {
            pemsao.logfiles = "pemsao.log"
            }
	pemsao.plotter = "stdplot"
	pemsao.dispmode = 2
	pemsao.vel_plot = "emission"
#	pemsao.curmode = curmode
	pemsao.curmode = no
	pemsao.dispem = yes
	pemsao.dispabs = no
	pemsao.ablines = "ablines.dat"
	pemsao.obj_plot = no
	pemsao.contsub_plot = no
	pemsao.debug = debug
	pemsao.nsum = 1
	pemsao.cursor = ""

# Compute shift of selected line using EMSAO
	if (db) {
	    dpar pemsao
	    }
	pemsao (imin)

# Compute mean pixel shift for selected emission line and read it back
	if (median) {
	    shift = pemsao.medpix
	    }
	else {
	    shift = pemsao.meanpix
	    }

# If pixel shift was not computed, return
	if (shift == 0.0) {
	    print ("HPIXSHIFT1: No shift returned from PEMSAO")
	    return
	    }

# Move original file to Noshift directory, if requested
	if (save) {
	    if (!access ("Noshift/")) {
		mkdir ("Noshift")
		}
	    newfile = "Noshift/" // imin
	    if (access (newfile)) {
		imdelete (newfile)
		}
	    copy (imin, "Noshift")
	    if (vb) {
		print ("HLIN1: Nonlinear ", imin, " => Noshift/")
		}
	    }
	else {
	    delete (imin)
	    }


# Add pixel shift to header
	gettime ()
	history = gettime.date // " " // gettime.time // " "
	history = history // lname // ": " // shift // " pix shift, aps " // aprange
	if (median) {
	    history = history // " median"
	    }
	else {
	    history = history // " mean"
	    }
	if (vb) {
	    print ("HPIXSHIFT1: ", imin, ": ", history)
	    }
	hedit (imin, "HPIXSHIF", history, add=yes, verify=no, show=db)
	cshift = shift
	hedit (imin, "PIXSHIFT", cshift, add=yes, verify=no, show=db)

	return

	end

# May 20 2005	Task to compute night sky spectrum shifts
# May 26 2005	Identify line from which to compute shift by its number
# May 27 2005	Add emlines = file of line positions, linedir = directory
# Jun  1 2005	Add apertures to HPIXSHIF keyword value
# Jun  6 2005	Add line name to HPIXSHIF keyword value
# Jun 23 2005	Check for HPIXSHIF keyword before doing anything else
# Jun 24 2005	Add mean or median to history
# Jun 24 2005	Return with error message if no EMSAO output file is found
# Jun 27 2005	Add option to save image without shift in header
# Jun 30 2005	Load wcstools and rvsao in hectospec.cl
# Jun 30 2005	Set emsao.verbose=yes always; plot only if debug and verbose

# Mar 30 2007	Add curmode to test for fitting problems (comment out, too)
# Mar 30 2007	Load rvsao and wcstools packages if necessary

# May 22 2007	Check for package wcstools instead of task getcol

# Sep 15 2008	Use single-line files if possible
# Sep 16 2008	Use pemsao.medpix for median and pemsao.meanpix for mean shift
