#!/bin/env ksh

# idcombine: Combine per-chip fiber assignment files into per-exposure file
# By Doug Mink, December 9, 2003

integer nfiles
typeset token

if test -z $1
    then
    echo "idcombine: Combine per-chip fiber assignment files into per-exposure file"

    echo "        idcombine [-v] <1st ID file> <2nd ID file> <output file>"
    echo "        -v: list progress"
    exit
    fi
v=
if test $1 = "-v"
    then
    #echo verbose mode
    v=true
    if test $idfile
	then
	echo Object name and position from $idfile
	fi
    shift
fi

idfile1=$1
idfile2=$2
idfile=$3
wcsdir=/data/oir/wcs/bin.solaris

# Move info from first chip's ID file directly to output file
$wcsdir/gettab $idfile1 aperture beam object ra dec target fiber platex platey | $wcsdir/getcol -t stdin 1-9 >$idfile
if test $v
    then
    echo IDCOMBINE: Chip 1 ID info copied to $idfile
    fi

# Count apertures on first chip
nap1=`$wcsdir/gettab $idfile1 aperture | wc -l | getcol stdin 1` 
if test $v
    then
    echo IDCOMBINE: $nap1 apertures on Chip 1
    fi

# Move info from second chip's ID file directly to output file, adding nap
#echo $wcsdir/gettab $idfile2 aperture beam object ra dec target fiber platex platey \| $wcsdir/getcol -t stdin 1+$nap1. 2-9 | getcol -t stdin 9 1-8
$wcsdir/gettab $idfile2 aperture beam object ra dec target fiber platex platey | $wcsdir/getcol -t stdin 1+$nap1. 2-9 | getcol -t stdin 9 1-8 >> $idfile
if test $v
    then
    echo IDCOMBINE: Chip 2 ID info copied to $idfile
    fi

# Dec  9 2003	New program
# Mar  3 2008	Change path to ksh
