# XCPIX1 - Piecewise cross-correlate a spectrum against a template in pixel
#          space using MKXCTEMP and XCSAO, make new cal file id file, and
#          reidentify
# February 10, 2004
# By Doug Mink, SAO Telescope Data Center

procedure xcpix1 (object)

string	object		{prompt="Object spectrum file"}

int	objap=0		{prompt="Object Aperture"}
string	eventemp	{prompt="Template spectrum file for even apertures"}
string	oddtemp		{prompt="Template spectrum file for odd apertures"}
string	ttemplate	{prompt="Shifted and stretched template file"}
int	firstpix=150	{prompt="Number of first pixel to correlate"}
int	lastpix=4150	{prompt="Number of last pixel to correlate"}
int	npix=200	{prompt="Number of pixels to correlate"}
int	nbin=256	{prompt="Number of rebinned pixels to correlate"}
int	intpix=100	{prompt="Interval between correlation centers in pixels"}
bool	intxcor=no	{prompt="Run cross-correlations interactively (y or n)"}
real	ndev=0		{prompt="Reject past this number of standard deviations"}
real	rmin=2.0	{prompt="Reject segment cross-correlations with R < this"}
bool	intreref=no	{prompt="Run reref fitting function interactively (y or n)"}
bool	intdisp=no	{prompt="Run reidentify function interactively (y or n)"}
bool	debug=no	{prompt="Print debugging information (y or n)"}

begin
	char arg
	string objsect
	string templist	# File to which to write list of segments
	string xcout	# File to which to write xcsao output
	string objfile, tempfile, ttempfile, ttempfile1
	string dbfile	# id file created by reref
	string dbfile1	# reref output id file renamed to match correlated file
	string dbtemp
	int ap, iap

	real constant
	real nd
	int im

	if (ndev < 1) {
	    nd = 10
	    }
	else {
	    nd = ndev
	    }
	objfile = object
	ap = objap
	if (ap > 0) {
	    objsect = objfile // "[*," // ap // "]"
	    }
	else {
	    objsect = objfile
	    }
	iap = ap / 2
	iap = iap * 2
	if (ap == iap) {
	    tempfile = eventemp
	    }
	else {
	    tempfile = oddtemp
	    }
	ttempfile = ttemplate

# Make list of templates npix wide every intpix pixels
	templist = mktemp ("xcpix")
	mkxctemp (tempfile,templist=templist,firstpix=firstpix,lastpix=lastpix,npix=npix,intpix=intpix)

# Cross-correlate spectrum against template, producing output in xcout
	rvsao (motd=no)
	arg = "@" // templist
	xcout = mktemp ("xcsao")
	if (ap > 0) {
	    xcsao.specnum = ap
	    }
	else {
	    xcsao.specnum = 0
	    }
	xcsao (objfile,templates=arg,correlate="pixel",tempnum=0,ncols=nbin,report_mode=16,logfile=xcout,curmode=intxcor,debug=debug)
	if (debug) {
	    print ("xcsao output written to ",xcout)
	    }
	delete (templist)

# If spectrum file does not exist for shifted template, create it
	if (!access (ttempfile)) {
	    ttempfile1 = ttempfile // ".fits"
	    if (!access (ttempfile1)) {
		printf ("Fake template file %s does not exist\n", ttempfile)
		imcopy (tempfile, ttempfile)
		}
	    }
	else if (debug) {
	    printf ("Fake template file is %s\n",ttempfile)
	    }

# If shifted template database file exists, delete it
	dbtemp = "database/id" // ttempfile
	if (access (dbtemp)) {
	    delete (dbtemp)
	    }

# Read pixels and shifts from xcout and fit a curve to them, then make
# a new database/id file with modified pixel positions for lines
#	print ("reref (",objfile,",", tempfile,",",ttempfile,",rmin=",rmin,",smin=",nd,",shiftfile=",xcout,",verbose=",verbose,",debug=",debug)
	reref (objfile, tempfile, ttempfile, apout=ap, rmin=rmin, smin=nd, sinteract=intreref, dinteract=intreref, shiftfile=xcout, verbose=verbose, debug=debug)
#	print "Ready to run reidentify"

# Identify lines in new spectrum using reidentify and new id file
	reidentify.reference = ttempfile
	reidentify.images = objfile
	reidentify.answer = "YES"
#	reidentify.crval = ""
#	reidentify.cdelt = ""
	reidentify.interactive = intdisp
	reidentify.section = objsect
	reidentify.newaps = no
	reidentify.override = yes
	reidentify.refit = yes
	reidentify.search = INDEF
	reidentify.nlost = 10
	reidentify.cradius = 5.
	reidentify.threshold = 100.
	reidentify.addfeatures = yes
	reidentify.coordlist = "hear.dat"
	reidentify.match = -3.
	reidentify.maxfeatures = 200
	reidentify.minsep = 2.
	reidentify.database = "database"
	reidentify.logfiles = "reidlog"
	reidentify.verbose = verbose
	reidentify (ttempfile, objsect)

# Append id file from cross-correlation to alternative id file
	dbfile = "database/id" // ttempfile
	dbfile1 = "database/id" // objfile // "x"
	if (debug) {
	    printf ("%s -> %s\n", dbfile, dbfile1)
	    }
	cat (dbfile, >>dbfile1)

	delete (xcout)
end

# Dec 19 2002	New task using reref instead of inline linear fit
# Apr 24 2003	Debug
# Apr 28 2003	Delete id file before reref creates a new one
# Jun  3 2003	Add ability to read multispec files
# Jun 30 2003	Turn into xcpix1 so xcpix can process multiple files
# Jun 30 2003	Rename new id file instead of deleting it
# Jun 30 2003	Use objsect instead of objfile when aperture is needed
# Jul  9 2003	Add rmin and separate intdisp and intxcor

# Feb 10 2004	Replace single template with odd and even aperture templates
# Feb 10 2004	Add reref interactive parameter
