#!/data/oiropt/starbase3.2.8/tawk -f

# get initial header - "begin" to "features"
# save table part
# get final header - fit and coefficients
# add to header apID, rms, min, max, nlines, nrej 
#
# print out header lines
# print out table lines
# go to next Aperture

 BEGIN { aptab["apid"] = 1; REV="1.0" }
 /^\#/         { aptab["date"]     = sprintf("Date\t%s", substr($0,2)) }
 /^\tid/       { aptab["file"]     = sprintf("%s\t%s",$1,$2) }
 /^\ttask/     { aptab["task"]     = sprintf("%s\t%s",$1,$2) }
 /^\timage/    { aptab["image"]    = sprintf("%s\t%s",$1,$2) }
 /^\taplow/    { aptab["aplow"]    = sprintf("%s\t%s",$1,$2) }
 /^\taphigh/   { aptab["aphigh"]   = sprintf("%s\t%s",$1,$2) }
 /^\tunits/    { aptab["units"]    = sprintf("%s\t%s",$1,$2) }
 /^\taperture/ { aptab["aperture"] = sprintf("%s\t%s",$1,$2) 
                 aptab["apid"]     = $2 + 0 }
 /^\tfeatures/ { 
     aptab["features"] = sprintf("%s\t%s",$1,$2) 
     nlines = $2
     for(i=1; i<=nlines; i+=1) {
       getline
       calib["pix" i] = $1    # pixel
       calib["obs" i] = $2    # Obs lambda
       calib["lam" i] = $3    # table lambda
       calib["rej" i] = $6+0  # 1=incl, 0=rej.
       species = $7
       for(j=8; j<=NF; j++) { species = species " " $j }
       sub("AI","ArI", species)
       calib["LID" i] = species

       resid = $2 - $3
       calib["res" i] = resid

       if(i == 1) {
         aptab["fw"] = sprintf("fwidth\t%s",$4)   # feature width?
         aptab["ns"] = sprintf("nsum\t%s",$5)     # nsum in "spatial" dir
         nrej = 0
         resmax = -10.; resmaxLam = ""; resmaxLID =""; resmaxPix = 0.0
         resmin = 10.; resminLam = ""; resmimLID = ""; resminPix = 0.0
         resrms = 0.
         resmean= 0.   
         nrej   = 0
       }
       if ($6 == 0) { 
          nrej += 1
       } else {
         if(resid < resmin) { 
           resmin = resid; resminLID = species; resminLam = $3; resminPix = $1 }
         if(resid > resmax) {
           resmax = resid; resmaxLID = species; resmaxLam = $3; resmaxPix = $1 }
         resrms  += resid * resid
         resmean += resid
       }
     }
     nused   = nlines - nrej
     resrms  = sqrt(resrms / nused);
     resmean = resmean / nused
 }
 /^\tfunction/     { aptab["function"]    = sprintf("%s\t%s",$1,$2) }
 /^\torder/        { aptab["order"]       = sprintf("%s\t%s",$1,$2) }
 /^\tsample/       { aptab["sample"]      = sprintf("%s\t%s",$1,$2) }
 /^\tnaverage/     { aptab["naverage"]    = sprintf("%s\t%s",$1,$2) }
 /^\tniterate/     { aptab["niterate"]    = sprintf("%s\t%s",$1,$2) }
 /^\tlow_reject/   { aptab["low_rej"]     = sprintf("%s\t%s",$1,$2) }
 /^\thigh_reject/  { aptab["high_rej"]    = sprintf("%s\t%s",$1,$2) }
 /^\tgrow/         { aptab["grow"]        = sprintf("%s\t%s",$1,$2) }
 /^\tcoefficients/ { 
       aptab["ncoeff"]      = sprintf("%s\t%s",$1,$2) 
       ncoeffs=$2
       for(i=1; i<=ncoeffs; i++) {
         getline
         coeff[i] = $NF
       }
  }
  /^$/ { # print the table in STARBASE format 
    if(first != 0) { printf "\f"}
    first += 1  
# output header stuff
    printf("AP%d\n", aptab["apid"])
    printf("Aperture\t%d\n", aptab["apid"])
    printf("converter\tdata2db.awk_Rev%s\n", REV)
    printf("%s\n",   aptab["date"])
    printf("%s\n",   aptab["file"])
    printf("%s\n",   aptab["task"])
    printf("%s\n",   aptab["image"])
    printf("%s\n",   aptab["aplow"])
    printf("%s\n",   aptab["aphigh"])
    printf("%s\n",   aptab["units"])
    printf("%s\n",   aptab["sample"])
    printf("%s\n",   aptab["fw"])
    printf("%s\n",   aptab["ns"])
    printf("%s\n",   aptab["niterate"])
    printf("%s\n",   aptab["low_rej"])
    printf("%s\n",   aptab["high_rej"])
    printf("%s\n",   aptab["grow"])

    printf("%s\n",   aptab["ncoeff"])
    printf("%s\n",   aptab["function"])
    printf("%s\n",   aptab["order"])
    printf("pixlow\t%s\n",  coeff[3])
    printf("pixhigh\t%s\n", coeff[4])
    for(i=5; i<=ncoeffs; i++) {
       printf("coef_%02d\t%s\n",i-4,coeff[i]) 
    }
    printf("%s\t%.2e\n", "resmean",   resmean)
    printf("%s\t%.4f\n", "resrms",    resrms )
    printf("%s\t%.4f\n", "resmin",    resmin)
    printf("%s\t%s\n",   "resminPix", resminPix)
    printf("%s\t%s\n",   "resminLam", resminLam)
    printf("%s\t%s\n",   "resminLID", resminLID)
    printf("%s\t%.4f\n", "resmax",    resmax)
    printf("%s\t%s\n",   "resmaxPix", resmaxPix)
    printf("%s\t%s\n",   "resmaxLam", resmaxLam)
    printf("%s\t%s\n",   "resmaxLID", resmaxLID)
    printf("%s\t%d\n",   "nlines",    nlines)
    printf("%s\t%d\n",   "nreject",   nrej)
    printf("%s\t%d\n",   "nused",     nused)

  # Output line table
    printf("\nPixel\tObsLam\tTableLam\tResid\tInc\tLineID\n")
    printf("-\t-\t-\t-\t-\t-\n")
    for(i=1; i<=nlines; i+=1) {
      printf("%s\t%s\t%s\t%.3f\t%s\t%s\n", 
             calib["pix" i], calib["obs" i], calib["lam" i], calib["res" i],
             calib["rej" i], calib["LID" i])
    }
  }

